# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
from typing import Any, AsyncIterable, AsyncIterator, Callable, Dict, IO, Optional, TypeVar, Union, cast, overload
import urllib.parse

from azure.core import AsyncPipelineClient
from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.async_arm_polling import AsyncARMPolling

from ... import models as _models
from ..._utils.serialization import Deserializer, Serializer
from ...operations._registries_operations import (
    build_check_name_availability_request,
    build_create_request,
    build_delete_request,
    build_generate_credentials_request,
    build_get_private_link_resource_request,
    build_get_request,
    build_import_image_request,
    build_list_by_resource_group_request,
    build_list_credentials_request,
    build_list_private_link_resources_request,
    build_list_request,
    build_list_usages_request,
    build_regenerate_credential_request,
    build_update_request,
)
from .._configuration import ContainerRegistryManagementClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class RegistriesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.containerregistry.v2022_02_01_preview.aio.ContainerRegistryManagementClient`'s
        :attr:`registries` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: ContainerRegistryManagementClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")
        self._api_version = input_args.pop(0) if input_args else kwargs.pop("api_version")

    async def _import_image_initial(
        self,
        resource_group_name: str,
        registry_name: str,
        parameters: Union[_models.ImportImageParameters, IO[bytes]],
        **kwargs: Any
    ) -> AsyncIterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "ImportImageParameters")

        _request = build_import_image_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def begin_import_image(
        self,
        resource_group_name: str,
        registry_name: str,
        parameters: _models.ImportImageParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[None]:
        """Copies an image to this container registry from the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param parameters: The parameters specifying the image to copy and the source container
         registry. Required.
        :type parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.ImportImageParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_import_image(
        self,
        resource_group_name: str,
        registry_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[None]:
        """Copies an image to this container registry from the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param parameters: The parameters specifying the image to copy and the source container
         registry. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def begin_import_image(
        self,
        resource_group_name: str,
        registry_name: str,
        parameters: Union[_models.ImportImageParameters, IO[bytes]],
        **kwargs: Any
    ) -> AsyncLROPoller[None]:
        """Copies an image to this container registry from the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param parameters: The parameters specifying the image to copy and the source container
         registry. Is either a ImportImageParameters type or a IO[bytes] type. Required.
        :type parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.ImportImageParameters or IO[bytes]
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._import_image_initial(
                resource_group_name=resource_group_name,
                registry_name=registry_name,
                parameters=parameters,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: AsyncPollingMethod = cast(AsyncPollingMethod, AsyncARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    @overload
    async def check_name_availability(
        self,
        registry_name_check_request: _models.RegistryNameCheckRequest,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RegistryNameStatus:
        """Checks whether the container registry name is available for use. The name must contain only
        alphanumeric characters, be globally unique, and between 5 and 50 characters in length.

        :param registry_name_check_request: The object containing information for the availability
         request. Required.
        :type registry_name_check_request:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryNameCheckRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: RegistryNameStatus or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryNameStatus
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def check_name_availability(
        self, registry_name_check_request: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.RegistryNameStatus:
        """Checks whether the container registry name is available for use. The name must contain only
        alphanumeric characters, be globally unique, and between 5 and 50 characters in length.

        :param registry_name_check_request: The object containing information for the availability
         request. Required.
        :type registry_name_check_request: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: RegistryNameStatus or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryNameStatus
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def check_name_availability(
        self, registry_name_check_request: Union[_models.RegistryNameCheckRequest, IO[bytes]], **kwargs: Any
    ) -> _models.RegistryNameStatus:
        """Checks whether the container registry name is available for use. The name must contain only
        alphanumeric characters, be globally unique, and between 5 and 50 characters in length.

        :param registry_name_check_request: The object containing information for the availability
         request. Is either a RegistryNameCheckRequest type or a IO[bytes] type. Required.
        :type registry_name_check_request:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryNameCheckRequest or IO[bytes]
        :return: RegistryNameStatus or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryNameStatus
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.RegistryNameStatus] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(registry_name_check_request, (IOBase, bytes)):
            _content = registry_name_check_request
        else:
            _json = self._serialize.body(registry_name_check_request, "RegistryNameCheckRequest")

        _request = build_check_name_availability_request(
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("RegistryNameStatus", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list(self, **kwargs: Any) -> AsyncIterable["_models.Registry"]:
        """Lists all the container registries under the specified subscription.

        :return: An iterator like instance of either Registry or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[_models.RegistryListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_request(
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("RegistryListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def list_by_resource_group(self, resource_group_name: str, **kwargs: Any) -> AsyncIterable["_models.Registry"]:
        """Lists all the container registries under the specified resource group.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :return: An iterator like instance of either Registry or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[_models.RegistryListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_by_resource_group_request(
                    resource_group_name=resource_group_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("RegistryListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def get(self, resource_group_name: str, registry_name: str, **kwargs: Any) -> _models.Registry:
        """Gets the properties of the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :return: Registry or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[_models.Registry] = kwargs.pop("cls", None)

        _request = build_get_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Registry", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    async def _create_initial(
        self, resource_group_name: str, registry_name: str, registry: Union[_models.Registry, IO[bytes]], **kwargs: Any
    ) -> AsyncIterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(registry, (IOBase, bytes)):
            _content = registry
        else:
            _json = self._serialize.body(registry, "Registry")

        _request = build_create_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def begin_create(
        self,
        resource_group_name: str,
        registry_name: str,
        registry: _models.Registry,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.Registry]:
        """Creates a container registry with the specified parameters.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param registry: The parameters for creating a container registry. Required.
        :type registry: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either Registry or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_create(
        self,
        resource_group_name: str,
        registry_name: str,
        registry: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.Registry]:
        """Creates a container registry with the specified parameters.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param registry: The parameters for creating a container registry. Required.
        :type registry: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either Registry or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def begin_create(
        self, resource_group_name: str, registry_name: str, registry: Union[_models.Registry, IO[bytes]], **kwargs: Any
    ) -> AsyncLROPoller[_models.Registry]:
        """Creates a container registry with the specified parameters.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param registry: The parameters for creating a container registry. Is either a Registry type or
         a IO[bytes] type. Required.
        :type registry: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry or IO[bytes]
        :return: An instance of AsyncLROPoller that returns either Registry or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Registry] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._create_initial(
                resource_group_name=resource_group_name,
                registry_name=registry_name,
                registry=registry,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("Registry", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: AsyncPollingMethod = cast(AsyncPollingMethod, AsyncARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[_models.Registry].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[_models.Registry](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )

    async def _delete_initial(
        self, resource_group_name: str, registry_name: str, **kwargs: Any
    ) -> AsyncIterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        _request = build_delete_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def begin_delete(self, resource_group_name: str, registry_name: str, **kwargs: Any) -> AsyncLROPoller[None]:
        """Deletes a container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._delete_initial(
                resource_group_name=resource_group_name,
                registry_name=registry_name,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: AsyncPollingMethod = cast(AsyncPollingMethod, AsyncARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    async def _update_initial(
        self,
        resource_group_name: str,
        registry_name: str,
        registry_update_parameters: Union[_models.RegistryUpdateParameters, IO[bytes]],
        **kwargs: Any
    ) -> AsyncIterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(registry_update_parameters, (IOBase, bytes)):
            _content = registry_update_parameters
        else:
            _json = self._serialize.body(registry_update_parameters, "RegistryUpdateParameters")

        _request = build_update_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def begin_update(
        self,
        resource_group_name: str,
        registry_name: str,
        registry_update_parameters: _models.RegistryUpdateParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.Registry]:
        """Updates a container registry with the specified parameters.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param registry_update_parameters: The parameters for updating a container registry. Required.
        :type registry_update_parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryUpdateParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either Registry or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_update(
        self,
        resource_group_name: str,
        registry_name: str,
        registry_update_parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.Registry]:
        """Updates a container registry with the specified parameters.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param registry_update_parameters: The parameters for updating a container registry. Required.
        :type registry_update_parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either Registry or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def begin_update(
        self,
        resource_group_name: str,
        registry_name: str,
        registry_update_parameters: Union[_models.RegistryUpdateParameters, IO[bytes]],
        **kwargs: Any
    ) -> AsyncLROPoller[_models.Registry]:
        """Updates a container registry with the specified parameters.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param registry_update_parameters: The parameters for updating a container registry. Is either
         a RegistryUpdateParameters type or a IO[bytes] type. Required.
        :type registry_update_parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryUpdateParameters or IO[bytes]
        :return: An instance of AsyncLROPoller that returns either Registry or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.Registry]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Registry] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._update_initial(
                resource_group_name=resource_group_name,
                registry_name=registry_name,
                registry_update_parameters=registry_update_parameters,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("Registry", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: AsyncPollingMethod = cast(AsyncPollingMethod, AsyncARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[_models.Registry].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[_models.Registry](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )

    @distributed_trace_async
    async def list_usages(
        self, resource_group_name: str, registry_name: str, **kwargs: Any
    ) -> _models.RegistryUsageListResult:
        """Gets the quota usages for the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :return: RegistryUsageListResult or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryUsageListResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[_models.RegistryUsageListResult] = kwargs.pop("cls", None)

        _request = build_list_usages_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("RegistryUsageListResult", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list_private_link_resources(
        self, resource_group_name: str, registry_name: str, **kwargs: Any
    ) -> AsyncIterable["_models.PrivateLinkResource"]:
        """Lists the private link resources for a container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :return: An iterator like instance of either PrivateLinkResource or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.containerregistry.v2022_02_01_preview.models.PrivateLinkResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[_models.PrivateLinkResourceListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_private_link_resources_request(
                    resource_group_name=resource_group_name,
                    registry_name=registry_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("PrivateLinkResourceListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def get_private_link_resource(
        self, resource_group_name: str, registry_name: str, group_name: str, **kwargs: Any
    ) -> _models.PrivateLinkResource:
        """Gets a private link resource by a specified group name for a container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param group_name: The name of the private link resource. Required.
        :type group_name: str
        :return: PrivateLinkResource or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.PrivateLinkResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[_models.PrivateLinkResource] = kwargs.pop("cls", None)

        _request = build_get_private_link_resource_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            group_name=group_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("PrivateLinkResource", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def list_credentials(
        self, resource_group_name: str, registry_name: str, **kwargs: Any
    ) -> _models.RegistryListCredentialsResult:
        """Lists the login credentials for the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :return: RegistryListCredentialsResult or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryListCredentialsResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        cls: ClsType[_models.RegistryListCredentialsResult] = kwargs.pop("cls", None)

        _request = build_list_credentials_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("RegistryListCredentialsResult", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def regenerate_credential(
        self,
        resource_group_name: str,
        registry_name: str,
        regenerate_credential_parameters: _models.RegenerateCredentialParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RegistryListCredentialsResult:
        """Regenerates one of the login credentials for the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param regenerate_credential_parameters: Specifies name of the password which should be
         regenerated -- password or password2. Required.
        :type regenerate_credential_parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegenerateCredentialParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: RegistryListCredentialsResult or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryListCredentialsResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def regenerate_credential(
        self,
        resource_group_name: str,
        registry_name: str,
        regenerate_credential_parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RegistryListCredentialsResult:
        """Regenerates one of the login credentials for the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param regenerate_credential_parameters: Specifies name of the password which should be
         regenerated -- password or password2. Required.
        :type regenerate_credential_parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: RegistryListCredentialsResult or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryListCredentialsResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def regenerate_credential(
        self,
        resource_group_name: str,
        registry_name: str,
        regenerate_credential_parameters: Union[_models.RegenerateCredentialParameters, IO[bytes]],
        **kwargs: Any
    ) -> _models.RegistryListCredentialsResult:
        """Regenerates one of the login credentials for the specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param regenerate_credential_parameters: Specifies name of the password which should be
         regenerated -- password or password2. Is either a RegenerateCredentialParameters type or a
         IO[bytes] type. Required.
        :type regenerate_credential_parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegenerateCredentialParameters or
         IO[bytes]
        :return: RegistryListCredentialsResult or the result of cls(response)
        :rtype: ~azure.mgmt.containerregistry.v2022_02_01_preview.models.RegistryListCredentialsResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.RegistryListCredentialsResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(regenerate_credential_parameters, (IOBase, bytes)):
            _content = regenerate_credential_parameters
        else:
            _json = self._serialize.body(regenerate_credential_parameters, "RegenerateCredentialParameters")

        _request = build_regenerate_credential_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("RegistryListCredentialsResult", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    async def _generate_credentials_initial(
        self,
        resource_group_name: str,
        registry_name: str,
        generate_credentials_parameters: Union[_models.GenerateCredentialsParameters, IO[bytes]],
        **kwargs: Any
    ) -> AsyncIterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(generate_credentials_parameters, (IOBase, bytes)):
            _content = generate_credentials_parameters
        else:
            _json = self._serialize.body(generate_credentials_parameters, "GenerateCredentialsParameters")

        _request = build_generate_credentials_request(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def begin_generate_credentials(
        self,
        resource_group_name: str,
        registry_name: str,
        generate_credentials_parameters: _models.GenerateCredentialsParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.GenerateCredentialsResult]:
        """Generate keys for a token of a specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param generate_credentials_parameters: The parameters for generating credentials. Required.
        :type generate_credentials_parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.GenerateCredentialsParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either GenerateCredentialsResult or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.GenerateCredentialsResult]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_generate_credentials(
        self,
        resource_group_name: str,
        registry_name: str,
        generate_credentials_parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.GenerateCredentialsResult]:
        """Generate keys for a token of a specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param generate_credentials_parameters: The parameters for generating credentials. Required.
        :type generate_credentials_parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of AsyncLROPoller that returns either GenerateCredentialsResult or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.GenerateCredentialsResult]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def begin_generate_credentials(
        self,
        resource_group_name: str,
        registry_name: str,
        generate_credentials_parameters: Union[_models.GenerateCredentialsParameters, IO[bytes]],
        **kwargs: Any
    ) -> AsyncLROPoller[_models.GenerateCredentialsResult]:
        """Generate keys for a token of a specified container registry.

        :param resource_group_name: The name of the resource group to which the container registry
         belongs. Required.
        :type resource_group_name: str
        :param registry_name: The name of the container registry. Required.
        :type registry_name: str
        :param generate_credentials_parameters: The parameters for generating credentials. Is either a
         GenerateCredentialsParameters type or a IO[bytes] type. Required.
        :type generate_credentials_parameters:
         ~azure.mgmt.containerregistry.v2022_02_01_preview.models.GenerateCredentialsParameters or
         IO[bytes]
        :return: An instance of AsyncLROPoller that returns either GenerateCredentialsResult or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.containerregistry.v2022_02_01_preview.models.GenerateCredentialsResult]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-02-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.GenerateCredentialsResult] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._generate_credentials_initial(
                resource_group_name=resource_group_name,
                registry_name=registry_name,
                generate_credentials_parameters=generate_credentials_parameters,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("GenerateCredentialsResult", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: AsyncPollingMethod = cast(AsyncPollingMethod, AsyncARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[_models.GenerateCredentialsResult].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[_models.GenerateCredentialsResult](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )



import os
import re
from glob import glob
from pathlib import Path
from typing import Set, List
from subprocess import check_call, call


def print_check(cmd: str):
    check_call(cmd, shell=True)


def print_exec(cmd: str):
    call(cmd, shell=True)


def step_into_rest_repo():
    os.chdir(str(Path(os.getcwd()) / 'azure-rest-api-specs'))


def git_clean():
    print_check('git checkout .')
    print_check('git clean -fd')
    print_check('git reset --hard HEAD')


class MultiApiConfigHelper:
    """
    This class can generate private SDK package
    """

    def __init__(self):
        self.service_name = ''
        self.tag = ''
        self.tag_files = {}  # Dict[str, List(str)]
        self.configured_files = set()  # Set[str]

    def get_input(self):
        print('Please commit your code before execute this script!!!')
        self.service_name = input('Please input your target service link(e.g. '
                                  '"https://github.com/Azure/azure-rest-api-specs/blob/main/specification/applicationinsights/resource-manager" '
                                  'or "applicationinsights"):\n')
        if 'resource-manager' in self.service_name:
            self.service_name = self.service_name.split('/')[-2]
        tag = input('Please input your target tag(e.g. "Readme Tag: package-2022-01-11" or "package-2022-01-11"):\n')
        self.tag = tag.split(':')[-1].strip()

    @staticmethod
    def checkout_main_branch():
        step_into_rest_repo()
        git_clean()
        usr = 'Azure'
        branch = 'main'
        print_exec(f'git remote add {usr} https://github.com/{usr}/azure-rest-api-specs.git')
        print_check(f'git fetch {usr} {branch}')
        print_check(f'git checkout {usr}/{branch}')

    def step_into_service_folder(self):
        root_path = os.getcwd()
        target_service_info = f'{root_path}/specification/{self.service_name}/resource-manager'
        result = glob(target_service_info)
        if len(result) == 0:
            raise Exception(f'do not find {target_service_info}')
        elif len(result) > 1:
            raise Exception(f'find multi target folder: {str(result)}')
        os.chdir(str(Path(result[0])))

    @staticmethod
    def judge_tag(line: str) -> bool:
        # ``` yaml $(tag) == 'package-2021-11-01'
        elements = ['```', 'tag', '==', 'package-']
        result = [re.search(element, line) for element in elements]
        return all(result)

    @staticmethod
    def get_tags(line: str) -> Set[str]:
        # extract 'package-2021-11-01' from "``` yaml $(tag) == 'package-2021-11-01'"
        result = re.findall("[\'\"][^\'^\"]+[\'\"]", line)
        if not result:
            raise Exception(f'Can not find valid tag name in {line}')
        return {item[1:-1] for item in result}

    @staticmethod
    def get_file_name(line: str) -> str:
        #   - Microsoft.Insights/stable/2015-05-01/aiOperations_API.json
        if '.json' in line and 'Microsoft.' in line:
            line = line.strip('\n ')
            line = line[1:] if line[0] == '-' else line
            return line.strip()
        return ''

    def get_tag_and_file(self, content: List[str], start_idx: int) -> int:
        tags_name = self.get_tags(content[start_idx])
        end_idx = start_idx
        for end_idx in range(start_idx + 1, len(content)):
            if '```' in content[end_idx]:
                break

        files = []
        for idx in range(start_idx + 1, end_idx):
            file_name = self.get_file_name(content[idx])
            if file_name:
                files.append(file_name)
        for tag_name in tags_name:
            if not self.tag_files.get(tag_name) and files:
                self.tag_files[tag_name] = files
        return end_idx + 1

    def get_all_tag_files(self):
        with open('readme.md', 'r') as file_in:
            content = file_in.readlines()

        i = 0
        while i < len(content):
            if self.judge_tag(content[i]):
                i = self.get_tag_and_file(content, i)
            else:
                i += 1

    @staticmethod
    def judge_config(line: str) -> bool:
        return 'batch' in line

    @staticmethod
    def get_configured_tag_name(line: str) -> str:
        # '  - tag: package-2017-10'
        if 'multiapiscript' in line:
            return ''
        return line.split('tag:')[-1].strip('\n ')

    def get_configured_tags(self, content: List[str], start_idx: int, tags: List[str]):
        end_idx = start_idx
        for end_idx in range(start_idx + 1, len(content)):
            if '```' in content[end_idx]:
                break

        for idx in range(start_idx + 1, end_idx):
            tag_name = self.get_configured_tag_name(content[idx])
            if tag_name:
                tags.append(tag_name)

    def get_configured_files(self):
        with open('readme.python.md', 'r') as file_in:
            content = file_in.readlines()

        configured_files = []
        configured_tags = []
        for i in range(len(content)):
            if self.judge_config(content[i]):
                self.get_configured_tags(content, i, configured_tags)
                break
        for tag in configured_tags:
            if tag not in self.tag_files:
                raise Exception(f'find {tag} in readme.python.md but it is not in readme.md!')
            configured_files.extend(self.tag_files[tag])

        self.configured_files = set(configured_files)

    def get_missing_files(self):
        missing_files = []
        if self.tag not in self.tag_files:
            raise Exception(f'Do not find \"{self.tag}\" in \"{self.service_name}/readme.md\"'
                            f'({str(self.tag_files.keys())})')
        print(f'find the following tiles in {self.tag}:')
        for file_name in self.tag_files[self.tag]:
            print(file_name)
            if file_name not in self.configured_files:
                missing_files.append('  - ' + file_name)

        print('Program done!')
        if missing_files:
            print(f'There are {len(missing_files)} files that are not configured in readme.python.md:')
            for file_name in missing_files:
                print(file_name)
        else:
            print('There are not missing files!!')
        os.system("pause")

    def compare_tag(self):
        self.step_into_service_folder()
        self.get_all_tag_files()
        self.get_configured_files()
        self.get_missing_files()

    def run(self):
        self.get_input()
        self.checkout_main_branch()
        self.compare_tag()


if __name__ == '__main__':
    instance = MultiApiConfigHelper()
    instance.run()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.compute.aio import ComputeManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestComputeManagementVirtualMachineScaleSetVMsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ComputeManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_reimage(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_reimage(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_reimage_all(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_reimage_all(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_approve_rolling_upgrade(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_approve_rolling_upgrade(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_deallocate(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_deallocate(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_update(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_update(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                parameters={
                    "location": "str",
                    "additionalCapabilities": {"hibernationEnabled": bool, "ultraSSDEnabled": bool},
                    "availabilitySet": {"id": "str"},
                    "diagnosticsProfile": {"bootDiagnostics": {"enabled": bool, "storageUri": "str"}},
                    "etag": "str",
                    "hardwareProfile": {"vmSize": "str", "vmSizeProperties": {"vCPUsAvailable": 0, "vCPUsPerCore": 0}},
                    "id": "str",
                    "identity": {
                        "principalId": "str",
                        "tenantId": "str",
                        "type": "str",
                        "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                    },
                    "instanceId": "str",
                    "instanceView": {
                        "assignedHost": "str",
                        "bootDiagnostics": {
                            "consoleScreenshotBlobUri": "str",
                            "serialConsoleLogBlobUri": "str",
                            "status": {
                                "code": "str",
                                "displayStatus": "str",
                                "level": "str",
                                "message": "str",
                                "time": "2020-02-20 00:00:00",
                            },
                        },
                        "computerName": "str",
                        "disks": [
                            {
                                "encryptionSettings": [
                                    {
                                        "diskEncryptionKey": {"secretUrl": "str", "sourceVault": {"id": "str"}},
                                        "enabled": bool,
                                        "keyEncryptionKey": {"keyUrl": "str", "sourceVault": {"id": "str"}},
                                    }
                                ],
                                "name": "str",
                                "statuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                            }
                        ],
                        "extensions": [
                            {
                                "name": "str",
                                "statuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                                "substatuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                                "type": "str",
                                "typeHandlerVersion": "str",
                            }
                        ],
                        "hyperVGeneration": "str",
                        "maintenanceRedeployStatus": {
                            "isCustomerInitiatedMaintenanceAllowed": bool,
                            "lastOperationMessage": "str",
                            "lastOperationResultCode": "str",
                            "maintenanceWindowEndTime": "2020-02-20 00:00:00",
                            "maintenanceWindowStartTime": "2020-02-20 00:00:00",
                            "preMaintenanceWindowEndTime": "2020-02-20 00:00:00",
                            "preMaintenanceWindowStartTime": "2020-02-20 00:00:00",
                        },
                        "osName": "str",
                        "osVersion": "str",
                        "placementGroupId": "str",
                        "platformFaultDomain": 0,
                        "platformUpdateDomain": 0,
                        "rdpThumbPrint": "str",
                        "statuses": [
                            {
                                "code": "str",
                                "displayStatus": "str",
                                "level": "str",
                                "message": "str",
                                "time": "2020-02-20 00:00:00",
                            }
                        ],
                        "vmAgent": {
                            "extensionHandlers": [
                                {
                                    "status": {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    },
                                    "type": "str",
                                    "typeHandlerVersion": "str",
                                }
                            ],
                            "statuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                            "vmAgentVersion": "str",
                        },
                        "vmHealth": {
                            "status": {
                                "code": "str",
                                "displayStatus": "str",
                                "level": "str",
                                "message": "str",
                                "time": "2020-02-20 00:00:00",
                            }
                        },
                    },
                    "latestModelApplied": bool,
                    "licenseType": "str",
                    "modelDefinitionApplied": "str",
                    "name": "str",
                    "networkProfile": {
                        "networkApiVersion": "str",
                        "networkInterfaceConfigurations": [
                            {
                                "name": "str",
                                "auxiliaryMode": "str",
                                "auxiliarySku": "str",
                                "deleteOption": "str",
                                "disableTcpStateTracking": bool,
                                "dnsSettings": {"dnsServers": ["str"]},
                                "dscpConfiguration": {"id": "str"},
                                "enableAcceleratedNetworking": bool,
                                "enableFpga": bool,
                                "enableIPForwarding": bool,
                                "ipConfigurations": [
                                    {
                                        "name": "str",
                                        "applicationGatewayBackendAddressPools": [{"id": "str"}],
                                        "applicationSecurityGroups": [{"id": "str"}],
                                        "loadBalancerBackendAddressPools": [{"id": "str"}],
                                        "primary": bool,
                                        "privateIPAddressVersion": "str",
                                        "publicIPAddressConfiguration": {
                                            "name": "str",
                                            "deleteOption": "str",
                                            "dnsSettings": {"domainNameLabel": "str", "domainNameLabelScope": "str"},
                                            "idleTimeoutInMinutes": 0,
                                            "ipTags": [{"ipTagType": "str", "tag": "str"}],
                                            "publicIPAddressVersion": "str",
                                            "publicIPAllocationMethod": "str",
                                            "publicIPPrefix": {"id": "str"},
                                            "sku": {"name": "str", "tier": "str"},
                                        },
                                        "subnet": {"id": "str"},
                                    }
                                ],
                                "networkSecurityGroup": {"id": "str"},
                                "primary": bool,
                            }
                        ],
                        "networkInterfaces": [{"deleteOption": "str", "id": "str", "primary": bool}],
                    },
                    "networkProfileConfiguration": {
                        "networkInterfaceConfigurations": [
                            {
                                "name": "str",
                                "auxiliaryMode": "str",
                                "auxiliarySku": "str",
                                "deleteOption": "str",
                                "disableTcpStateTracking": bool,
                                "dnsSettings": {"dnsServers": ["str"]},
                                "enableAcceleratedNetworking": bool,
                                "enableFpga": bool,
                                "enableIPForwarding": bool,
                                "ipConfigurations": [
                                    {
                                        "name": "str",
                                        "applicationGatewayBackendAddressPools": [{"id": "str"}],
                                        "applicationSecurityGroups": [{"id": "str"}],
                                        "loadBalancerBackendAddressPools": [{"id": "str"}],
                                        "loadBalancerInboundNatPools": [{"id": "str"}],
                                        "primary": bool,
                                        "privateIPAddressVersion": "str",
                                        "publicIPAddressConfiguration": {
                                            "name": "str",
                                            "deleteOption": "str",
                                            "dnsSettings": {"domainNameLabel": "str", "domainNameLabelScope": "str"},
                                            "idleTimeoutInMinutes": 0,
                                            "ipTags": [{"ipTagType": "str", "tag": "str"}],
                                            "publicIPAddressVersion": "str",
                                            "publicIPPrefix": {"id": "str"},
                                            "sku": {"name": "str", "tier": "str"},
                                        },
                                        "subnet": {"id": "str"},
                                    }
                                ],
                                "networkSecurityGroup": {"id": "str"},
                                "primary": bool,
                            }
                        ]
                    },
                    "osProfile": {
                        "adminPassword": "str",
                        "adminUsername": "str",
                        "allowExtensionOperations": bool,
                        "computerName": "str",
                        "customData": "str",
                        "linuxConfiguration": {
                            "disablePasswordAuthentication": bool,
                            "enableVMAgentPlatformUpdates": bool,
                            "patchSettings": {
                                "assessmentMode": "str",
                                "automaticByPlatformSettings": {
                                    "bypassPlatformSafetyChecksOnUserSchedule": bool,
                                    "rebootSetting": "str",
                                },
                                "patchMode": "str",
                            },
                            "provisionVMAgent": bool,
                            "ssh": {"publicKeys": [{"keyData": "str", "path": "str"}]},
                        },
                        "requireGuestProvisionSignal": bool,
                        "secrets": [
                            {
                                "sourceVault": {"id": "str"},
                                "vaultCertificates": [{"certificateStore": "str", "certificateUrl": "str"}],
                            }
                        ],
                        "windowsConfiguration": {
                            "additionalUnattendContent": [
                                {
                                    "componentName": "Microsoft-Windows-Shell-Setup",
                                    "content": "str",
                                    "passName": "OobeSystem",
                                    "settingName": "str",
                                }
                            ],
                            "enableAutomaticUpdates": bool,
                            "enableVMAgentPlatformUpdates": bool,
                            "patchSettings": {
                                "assessmentMode": "str",
                                "automaticByPlatformSettings": {
                                    "bypassPlatformSafetyChecksOnUserSchedule": bool,
                                    "rebootSetting": "str",
                                },
                                "enableHotpatching": bool,
                                "patchMode": "str",
                            },
                            "provisionVMAgent": bool,
                            "timeZone": "str",
                            "winRM": {"listeners": [{"certificateUrl": "str", "protocol": "str"}]},
                        },
                    },
                    "plan": {"name": "str", "product": "str", "promotionCode": "str", "publisher": "str"},
                    "protectionPolicy": {"protectFromScaleIn": bool, "protectFromScaleSetActions": bool},
                    "provisioningState": "str",
                    "resilientVMDeletionStatus": "str",
                    "resources": [
                        {
                            "autoUpgradeMinorVersion": bool,
                            "enableAutomaticUpgrade": bool,
                            "forceUpdateTag": "str",
                            "id": "str",
                            "instanceView": {
                                "name": "str",
                                "statuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                                "substatuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                                "type": "str",
                                "typeHandlerVersion": "str",
                            },
                            "location": "str",
                            "name": "str",
                            "protectedSettings": {},
                            "protectedSettingsFromKeyVault": {"secretUrl": "str", "sourceVault": {"id": "str"}},
                            "provisionAfterExtensions": ["str"],
                            "provisioningState": "str",
                            "publisher": "str",
                            "settings": {},
                            "suppressFailures": bool,
                            "tags": {"str": "str"},
                            "type": "str",
                            "typeHandlerVersion": "str",
                        }
                    ],
                    "securityProfile": {
                        "encryptionAtHost": bool,
                        "encryptionIdentity": {"userAssignedIdentityResourceId": "str"},
                        "proxyAgentSettings": {
                            "enabled": bool,
                            "imds": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                            "keyIncarnationId": 0,
                            "mode": "str",
                            "wireServer": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                        },
                        "securityType": "str",
                        "uefiSettings": {"secureBootEnabled": bool, "vTpmEnabled": bool},
                    },
                    "sku": {"capacity": 0, "name": "str", "tier": "str"},
                    "storageProfile": {
                        "alignRegionalDisksToVMZone": bool,
                        "dataDisks": [
                            {
                                "createOption": "str",
                                "lun": 0,
                                "caching": "str",
                                "deleteOption": "str",
                                "detachOption": "str",
                                "diskIOPSReadWrite": 0,
                                "diskMBpsReadWrite": 0,
                                "diskSizeGB": 0,
                                "image": {"uri": "str"},
                                "managedDisk": {
                                    "diskEncryptionSet": {"id": "str"},
                                    "id": "str",
                                    "securityProfile": {
                                        "diskEncryptionSet": {"id": "str"},
                                        "securityEncryptionType": "str",
                                    },
                                    "storageAccountType": "str",
                                },
                                "name": "str",
                                "sourceResource": {"id": "str"},
                                "toBeDetached": bool,
                                "vhd": {"uri": "str"},
                                "writeAcceleratorEnabled": bool,
                            }
                        ],
                        "diskControllerType": "str",
                        "imageReference": {
                            "communityGalleryImageId": "str",
                            "exactVersion": "str",
                            "id": "str",
                            "offer": "str",
                            "publisher": "str",
                            "sharedGalleryImageId": "str",
                            "sku": "str",
                            "version": "str",
                        },
                        "osDisk": {
                            "createOption": "str",
                            "caching": "str",
                            "deleteOption": "str",
                            "diffDiskSettings": {"option": "str", "placement": "str"},
                            "diskSizeGB": 0,
                            "encryptionSettings": {
                                "diskEncryptionKey": {"secretUrl": "str", "sourceVault": {"id": "str"}},
                                "enabled": bool,
                                "keyEncryptionKey": {"keyUrl": "str", "sourceVault": {"id": "str"}},
                            },
                            "image": {"uri": "str"},
                            "managedDisk": {
                                "diskEncryptionSet": {"id": "str"},
                                "id": "str",
                                "securityProfile": {
                                    "diskEncryptionSet": {"id": "str"},
                                    "securityEncryptionType": "str",
                                },
                                "storageAccountType": "str",
                            },
                            "name": "str",
                            "osType": "str",
                            "vhd": {"uri": "str"},
                            "writeAcceleratorEnabled": bool,
                        },
                    },
                    "tags": {"str": "str"},
                    "timeCreated": "2020-02-20 00:00:00",
                    "type": "str",
                    "userData": "str",
                    "vmId": "str",
                    "zones": ["str"],
                },
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_delete(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_delete(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_get(self, resource_group):
        response = await self.client.virtual_machine_scale_set_vms.get(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_get_instance_view(self, resource_group):
        response = await self.client.virtual_machine_scale_set_vms.get_instance_view(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_list(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.list(
            resource_group_name=resource_group.name,
            virtual_machine_scale_set_name="str",
            api_version="2024-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_power_off(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_power_off(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_restart(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_restart(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_start(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_start(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_redeploy(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_redeploy(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_retrieve_boot_diagnostics_data(self, resource_group):
        response = await self.client.virtual_machine_scale_set_vms.retrieve_boot_diagnostics_data(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_perform_maintenance(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_perform_maintenance(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_simulate_eviction(self, resource_group):
        response = await self.client.virtual_machine_scale_set_vms.simulate_eviction(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_attach_detach_data_disks(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_attach_detach_data_disks(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                parameters={
                    "dataDisksToAttach": [
                        {
                            "diskId": "str",
                            "caching": "str",
                            "deleteOption": "str",
                            "diskEncryptionSet": {"id": "str"},
                            "lun": 0,
                            "writeAcceleratorEnabled": bool,
                        }
                    ],
                    "dataDisksToDetach": [{"diskId": "str", "detachOption": "str"}],
                },
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_scale_set_vms_begin_run_command(self, resource_group):
        response = await (
            await self.client.virtual_machine_scale_set_vms.begin_run_command(
                resource_group_name=resource_group.name,
                vm_scale_set_name="str",
                instance_id="str",
                parameters={"commandId": "str", "parameters": [{"name": "str", "value": "str"}], "script": ["str"]},
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



#-------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#--------------------------------------------------------------------------
from . import KnownProfiles, ProfileDefinition

class InvalidMultiApiClientError(Exception):
    """If the mixin is not used with a compatible class.
    """
    pass

class MultiApiClientMixin(object):
    """Mixin that contains multi-api version profile management.

    To use this mixin, a client must define two class attributes:
    - LATEST_PROFILE : a ProfileDefinition correspond to latest profile
    - _PROFILE_TAG : a tag that filter a full profile for this particular client

    This should not be used directly and will only provide private methods.
    """

    def __init__(self, *args, **kwargs):
        # Consume "api_version" and "profile", to avoid sending them to base class
        api_version = kwargs.pop("api_version", None)
        profile = kwargs.pop("profile", KnownProfiles.default)

        # Can't do "super" call here all the time, or I would break old client with:
        # TypeError: object.__init__() takes no parameters
        # So I try to detect if I correctly use this class as a Mixin, or the messed-up
        # approach like before. If I think it's the messed-up old approach,
        # don't call super
        if args or "creds" in kwargs or "config" in kwargs:
            super(MultiApiClientMixin, self).__init__(*args, **kwargs)

        try:
            type(self).LATEST_PROFILE
        except AttributeError:
            raise InvalidMultiApiClientError("To use this mixin, main client MUST define LATEST_PROFILE class attribute")

        try:
            type(self)._PROFILE_TAG
        except AttributeError:
            raise InvalidMultiApiClientError("To use this mixin, main client MUST define _PROFILE_TAG class attribute")

        if api_version and profile is not KnownProfiles.default:
            raise ValueError("Cannot use api-version and profile parameters at the same time")

        if api_version:
            self.profile = ProfileDefinition({
                self._PROFILE_TAG: {
                    None: api_version
                }},
                self._PROFILE_TAG + " " + api_version
            )
        elif isinstance(profile, dict):
            self.profile = ProfileDefinition({
                self._PROFILE_TAG: profile,
                },
                self._PROFILE_TAG + " dict"
            )
            if api_version:
                self.profile._profile_dict[self._PROFILE_TAG][None] = api_version
        else:
            self.profile = profile

    def _get_api_version(self, operation_group_name):
        current_profile = self.profile
        if self.profile is KnownProfiles.default:
            current_profile = KnownProfiles.default.value.definition()

        if current_profile is KnownProfiles.latest:
            current_profile = self.LATEST_PROFILE
        elif isinstance(current_profile, KnownProfiles):
            current_profile = current_profile.value
        elif isinstance(current_profile, ProfileDefinition):
            pass  # I expect that
        else:
            raise ValueError("Cannot determine a ProfileDefinition from {}".format(self.profile))

        local_profile_dict = current_profile.get_profile_dict()
        if self._PROFILE_TAG not in local_profile_dict:
            raise ValueError("This profile doesn't define {}".format(self._PROFILE_TAG))

        local_profile = local_profile_dict[self._PROFILE_TAG]
        if operation_group_name in local_profile:
            return local_profile[operation_group_name]
        try:
            return local_profile[None]
        except KeyError:
            raise ValueError("This profile definition does not contain a default API version")



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: chat_client_sample.py
DESCRIPTION:
    These samples demonstrate create a chat client, get a chat thread client,
    create a chat thread, get a chat thread by id, list chat threads, delete
    a chat thread by id.
    You need to use azure.communication.configuration module to get user access
    token and user identity before run this sample

USAGE:
    python chat_client_sample.py
    Set the environment variables with your own values before running the sample:
    1) AZURE_COMMUNICATION_SERVICE_ENDPOINT - Communication Service endpoint url
    2) TOKEN - the user access token, from token_response.token
    3) USER_ID - the user id, from token_response.identity
"""


import os


class ChatClientSamples(object):
    from azure.communication.identity import CommunicationIdentityClient

    connection_string = os.environ.get("COMMUNICATION_SAMPLES_CONNECTION_STRING", None)
    if not connection_string:
        raise ValueError("Set COMMUNICATION_SAMPLES_CONNECTION_STRING env before run this sample.")

    identity_client = CommunicationIdentityClient.from_connection_string(connection_string)
    user = identity_client.create_user()
    tokenresponse = identity_client.get_token(user, scopes=["chat"])
    token = tokenresponse.token

    endpoint = os.environ.get("AZURE_COMMUNICATION_SERVICE_ENDPOINT", None)
    if not endpoint:
        raise ValueError("Set AZURE_COMMUNICATION_SERVICE_ENDPOINT env before run this sample.")

    _thread_id = None

    def create_chat_client(self):
        token = self.token
        endpoint = self.endpoint
        # [START create_chat_client]
        from azure.communication.chat import ChatClient, CommunicationTokenCredential

        # set `endpoint` to an existing ACS endpoint
        chat_client = ChatClient(endpoint, CommunicationTokenCredential(token))
        # [END create_chat_client]

    def create_thread(self):
        token = self.token
        endpoint = self.endpoint
        user = self.user
        # [START create_thread]
        from datetime import datetime

        from azure.communication.chat import (
            ChatClient,
            ChatParticipant,
            CommunicationUserIdentifier,
            CommunicationTokenCredential,
        )

        # set `endpoint` to an existing ACS endpoint
        chat_client = ChatClient(endpoint, CommunicationTokenCredential(token))

        topic = "test topic"
        participants = [ChatParticipant(identifier=user, display_name="name", share_history_time=datetime.utcnow())]

        # creates a new chat_thread everytime
        create_chat_thread_result = chat_client.create_chat_thread(topic, thread_participants=participants)

        # creates a new chat_thread if not exists
        idempotency_token = "b66d6031-fdcc-41df-8306-e524c9f226b8"  # unique identifier
        create_chat_thread_result_w_repeatability_id = chat_client.create_chat_thread(
            topic, thread_participants=participants, idempotency_token=idempotency_token
        )
        # [END create_thread]

        self._thread_id = create_chat_thread_result.chat_thread.id
        print("thread created, id: " + self._thread_id)

    def get_chat_thread_client(self):
        token = self.token
        endpoint = self.endpoint
        thread_id = self._thread_id

        # [START get_chat_thread_client]
        from azure.communication.chat import ChatClient, CommunicationTokenCredential

        # set `endpoint` to an existing ACS endpoint
        chat_client = ChatClient(endpoint, CommunicationTokenCredential(token))

        # set `thread_id` to an existing chat thread id
        chat_thread_client = chat_client.get_chat_thread_client(thread_id)
        # [END get_chat_thread_client]

        print("get_chat_thread_client succeeded with thread id: ", chat_thread_client.thread_id)

    def list_threads(self):
        token = self.token
        endpoint = self.endpoint

        # [START list_threads]
        from azure.communication.chat import ChatClient, CommunicationTokenCredential
        from datetime import datetime, timedelta

        # set `endpoint` to an existing ACS endpoint
        chat_client = ChatClient(endpoint, CommunicationTokenCredential(token))
        start_time = datetime.utcnow() - timedelta(days=2)
        chat_threads = chat_client.list_chat_threads(results_per_page=5, start_time=start_time)

        print("list_threads succeeded with results_per_page is 5, and were created since 2 days ago.")
        for chat_thread_item_page in chat_threads.by_page():
            for chat_thread_item in chat_thread_item_page:
                print("thread id:", chat_thread_item.id)
        # [END list_threads]

    def delete_thread(self):
        token = self.token
        endpoint = self.endpoint
        thread_id = self._thread_id
        # [START delete_thread]
        from azure.communication.chat import ChatClient, CommunicationTokenCredential

        # set `endpoint` to an existing ACS endpoint
        chat_client = ChatClient(endpoint, CommunicationTokenCredential(token))

        # set `thread_id` to an existing chat thread id
        chat_client.delete_chat_thread(thread_id)
        # [END delete_thread]

        print("delete_thread succeeded")

    def clean_up(self):
        print("cleaning up: deleting created user.")
        self.identity_client.delete_user(self.user)


if __name__ == "__main__":
    sample = ChatClientSamples()
    sample.create_chat_client()
    sample.create_thread()
    sample.get_chat_thread_client()
    sample.list_threads()
    sample.delete_thread()
    sample.clean_up()



# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, AsyncIterator, Callable, Dict, Optional, Type, TypeVar, Union, cast

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.async_arm_polling import AsyncARMPolling

from ... import models as _models
from ...operations._subnet_service_association_link_operations import build_delete_request

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class SubnetServiceAssociationLinkOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.containerinstance.aio.ContainerInstanceManagementClient`'s
        :attr:`subnet_service_association_link` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    async def _delete_initial(
        self, resource_group_name: str, virtual_network_name: str, subnet_name: str, **kwargs: Any
    ) -> AsyncIterator[bytes]:
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        _request = build_delete_request(
            resource_group_name=resource_group_name,
            virtual_network_name=virtual_network_name,
            subnet_name=subnet_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def begin_delete(
        self, resource_group_name: str, virtual_network_name: str, subnet_name: str, **kwargs: Any
    ) -> AsyncLROPoller[None]:
        """Delete container group virtual network association links.

        Delete container group virtual network association links. The operation does not delete other
        resources provided by the user.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param virtual_network_name: The name of the virtual network. Required.
        :type virtual_network_name: str
        :param subnet_name: The name of the subnet. Required.
        :type subnet_name: str
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._delete_initial(
                resource_group_name=resource_group_name,
                virtual_network_name=virtual_network_name,
                subnet_name=subnet_name,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: AsyncPollingMethod = cast(AsyncPollingMethod, AsyncARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.connectedcache import ConnectedCacheMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-connectedcache
# USAGE
    python isp_customers_delete_maximum_set_gen.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ConnectedCacheMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    client.isp_customers.begin_delete(
        resource_group_name="rgConnectedCache",
        customer_resource_name="hdontfoythjsaeyjhrakckgimgchxwzttbcnvntpvdsgeumxpgnjurptd",
    ).result()


# x-ms-original-file: 2023-05-01-preview/IspCustomers_Delete_MaximumSet_Gen.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import Activity
from ._models_py3 import ActivityListResult
from ._models_py3 import ActivityOutputType
from ._models_py3 import ActivityParameter
from ._models_py3 import ActivityParameterSet
from ._models_py3 import ActivityParameterValidationSet
from ._models_py3 import AdvancedSchedule
from ._models_py3 import AdvancedScheduleMonthlyOccurrence
from ._models_py3 import AgentRegistration
from ._models_py3 import AgentRegistrationKeys
from ._models_py3 import AgentRegistrationRegenerateKeyParameter
from ._models_py3 import AutomationAccount
from ._models_py3 import AutomationAccountCreateOrUpdateParameters
from ._models_py3 import AutomationAccountListResult
from ._models_py3 import AutomationAccountUpdateParameters
from ._models_py3 import AzureQueryProperties
from ._models_py3 import Certificate
from ._models_py3 import CertificateCreateOrUpdateParameters
from ._models_py3 import CertificateListResult
from ._models_py3 import CertificateUpdateParameters
from ._models_py3 import Connection
from ._models_py3 import ConnectionCreateOrUpdateParameters
from ._models_py3 import ConnectionListResult
from ._models_py3 import ConnectionType
from ._models_py3 import ConnectionTypeAssociationProperty
from ._models_py3 import ConnectionTypeCreateOrUpdateParameters
from ._models_py3 import ConnectionTypeListResult
from ._models_py3 import ConnectionUpdateParameters
from ._models_py3 import ContentHash
from ._models_py3 import ContentLink
from ._models_py3 import ContentSource
from ._models_py3 import Credential
from ._models_py3 import CredentialCreateOrUpdateParameters
from ._models_py3 import CredentialListResult
from ._models_py3 import CredentialUpdateParameters
from ._models_py3 import DeletedAutomationAccount
from ._models_py3 import DeletedAutomationAccountListResult
from ._models_py3 import Dimension
from ._models_py3 import DscCompilationJob
from ._models_py3 import DscCompilationJobCreateParameters
from ._models_py3 import DscCompilationJobListResult
from ._models_py3 import DscConfiguration
from ._models_py3 import DscConfigurationAssociationProperty
from ._models_py3 import DscConfigurationCreateOrUpdateParameters
from ._models_py3 import DscConfigurationListResult
from ._models_py3 import DscConfigurationParameter
from ._models_py3 import DscConfigurationUpdateParameters
from ._models_py3 import DscMetaConfiguration
from ._models_py3 import DscNode
from ._models_py3 import DscNodeConfiguration
from ._models_py3 import DscNodeConfigurationCreateOrUpdateParameters
from ._models_py3 import DscNodeConfigurationListResult
from ._models_py3 import DscNodeExtensionHandlerAssociationProperty
from ._models_py3 import DscNodeListResult
from ._models_py3 import DscNodeReport
from ._models_py3 import DscNodeReportListResult
from ._models_py3 import DscNodeUpdateParameters
from ._models_py3 import DscNodeUpdateParametersProperties
from ._models_py3 import DscReportError
from ._models_py3 import DscReportResource
from ._models_py3 import DscReportResourceNavigation
from ._models_py3 import EncryptionProperties
from ._models_py3 import EncryptionPropertiesIdentity
from ._models_py3 import ErrorResponse
from ._models_py3 import FieldDefinition
from ._models_py3 import GraphicalRunbookContent
from ._models_py3 import HybridRunbookWorker
from ._models_py3 import HybridRunbookWorkerCreateParameters
from ._models_py3 import HybridRunbookWorkerGroup
from ._models_py3 import HybridRunbookWorkerGroupCreateOrUpdateParameters
from ._models_py3 import HybridRunbookWorkerGroupsListResult
from ._models_py3 import HybridRunbookWorkerMoveParameters
from ._models_py3 import HybridRunbookWorkersListResult
from ._models_py3 import Identity
from ._models_py3 import Job
from ._models_py3 import JobCollectionItem
from ._models_py3 import JobCreateParameters
from ._models_py3 import JobListResultV2
from ._models_py3 import JobNavigation
from ._models_py3 import JobSchedule
from ._models_py3 import JobScheduleCreateParameters
from ._models_py3 import JobScheduleListResult
from ._models_py3 import JobStream
from ._models_py3 import JobStreamListResult
from ._models_py3 import Key
from ._models_py3 import KeyListResult
from ._models_py3 import KeyVaultProperties
from ._models_py3 import LinkedWorkspace
from ._models_py3 import LinuxProperties
from ._models_py3 import LogSpecification
from ._models_py3 import MetricSpecification
from ._models_py3 import Module
from ._models_py3 import ModuleCreateOrUpdateParameters
from ._models_py3 import ModuleErrorInfo
from ._models_py3 import ModuleListResult
from ._models_py3 import ModuleUpdateParameters
from ._models_py3 import NodeCount
from ._models_py3 import NodeCountProperties
from ._models_py3 import NodeCounts
from ._models_py3 import NonAzureQueryProperties
from ._models_py3 import Operation
from ._models_py3 import OperationDisplay
from ._models_py3 import OperationListResult
from ._models_py3 import OperationPropertiesFormatServiceSpecification
from ._models_py3 import PrivateEndpointConnection
from ._models_py3 import PrivateEndpointConnectionListResult
from ._models_py3 import PrivateEndpointProperty
from ._models_py3 import PrivateLinkResource
from ._models_py3 import PrivateLinkResourceListResult
from ._models_py3 import PrivateLinkServiceConnectionStateProperty
from ._models_py3 import ProxyResource
from ._models_py3 import PythonPackageCreateParameters
from ._models_py3 import PythonPackageUpdateParameters
from ._models_py3 import RawGraphicalRunbookContent
from ._models_py3 import Resource
from ._models_py3 import RunAsCredentialAssociationProperty
from ._models_py3 import Runbook
from ._models_py3 import RunbookAssociationProperty
from ._models_py3 import RunbookCreateOrUpdateDraftParameters
from ._models_py3 import RunbookCreateOrUpdateDraftProperties
from ._models_py3 import RunbookCreateOrUpdateParameters
from ._models_py3 import RunbookDraft
from ._models_py3 import RunbookDraftUndoEditResult
from ._models_py3 import RunbookListResult
from ._models_py3 import RunbookParameter
from ._models_py3 import RunbookUpdateParameters
from ._models_py3 import SUCScheduleProperties
from ._models_py3 import Schedule
from ._models_py3 import ScheduleAssociationProperty
from ._models_py3 import ScheduleCreateOrUpdateParameters
from ._models_py3 import ScheduleListResult
from ._models_py3 import ScheduleUpdateParameters
from ._models_py3 import Sku
from ._models_py3 import SoftwareUpdateConfiguration
from ._models_py3 import SoftwareUpdateConfigurationCollectionItem
from ._models_py3 import SoftwareUpdateConfigurationListResult
from ._models_py3 import SoftwareUpdateConfigurationMachineRun
from ._models_py3 import SoftwareUpdateConfigurationMachineRunListResult
from ._models_py3 import SoftwareUpdateConfigurationRun
from ._models_py3 import SoftwareUpdateConfigurationRunListResult
from ._models_py3 import SoftwareUpdateConfigurationRunTaskProperties
from ._models_py3 import SoftwareUpdateConfigurationRunTasks
from ._models_py3 import SoftwareUpdateConfigurationTasks
from ._models_py3 import SourceControl
from ._models_py3 import SourceControlCreateOrUpdateParameters
from ._models_py3 import SourceControlListResult
from ._models_py3 import SourceControlSecurityTokenProperties
from ._models_py3 import SourceControlSyncJob
from ._models_py3 import SourceControlSyncJobById
from ._models_py3 import SourceControlSyncJobCreateParameters
from ._models_py3 import SourceControlSyncJobListResult
from ._models_py3 import SourceControlSyncJobStream
from ._models_py3 import SourceControlSyncJobStreamById
from ._models_py3 import SourceControlSyncJobStreamsListBySyncJob
from ._models_py3 import SourceControlUpdateParameters
from ._models_py3 import Statistics
from ._models_py3 import StatisticsListResult
from ._models_py3 import SystemData
from ._models_py3 import TagSettingsProperties
from ._models_py3 import TargetProperties
from ._models_py3 import TaskProperties
from ._models_py3 import TestJob
from ._models_py3 import TestJobCreateParameters
from ._models_py3 import TrackedResource
from ._models_py3 import TypeField
from ._models_py3 import TypeFieldListResult
from ._models_py3 import UpdateConfiguration
from ._models_py3 import UpdateConfigurationNavigation
from ._models_py3 import Usage
from ._models_py3 import UsageCounterName
from ._models_py3 import UsageListResult
from ._models_py3 import UserAssignedIdentitiesProperties
from ._models_py3 import Variable
from ._models_py3 import VariableCreateOrUpdateParameters
from ._models_py3 import VariableListResult
from ._models_py3 import VariableUpdateParameters
from ._models_py3 import Watcher
from ._models_py3 import WatcherListResult
from ._models_py3 import WatcherUpdateParameters
from ._models_py3 import Webhook
from ._models_py3 import WebhookCreateOrUpdateParameters
from ._models_py3 import WebhookListResult
from ._models_py3 import WebhookUpdateParameters
from ._models_py3 import WindowsProperties

from ._automation_client_enums import AgentRegistrationKeyName
from ._automation_client_enums import AutomationAccountState
from ._automation_client_enums import AutomationKeyName
from ._automation_client_enums import AutomationKeyPermissions
from ._automation_client_enums import ContentSourceType
from ._automation_client_enums import CountType
from ._automation_client_enums import CreatedByType
from ._automation_client_enums import DscConfigurationState
from ._automation_client_enums import EncryptionKeySourceType
from ._automation_client_enums import GraphRunbookType
from ._automation_client_enums import GroupTypeEnum
from ._automation_client_enums import HttpStatusCode
from ._automation_client_enums import JobProvisioningState
from ._automation_client_enums import JobStatus
from ._automation_client_enums import JobStreamType
from ._automation_client_enums import LinuxUpdateClasses
from ._automation_client_enums import ModuleProvisioningState
from ._automation_client_enums import OperatingSystemType
from ._automation_client_enums import ProvisioningState
from ._automation_client_enums import ResourceIdentityType
from ._automation_client_enums import RunbookState
from ._automation_client_enums import RunbookTypeEnum
from ._automation_client_enums import ScheduleDay
from ._automation_client_enums import ScheduleFrequency
from ._automation_client_enums import SkuNameEnum
from ._automation_client_enums import SourceType
from ._automation_client_enums import StreamType
from ._automation_client_enums import SyncType
from ._automation_client_enums import TagOperators
from ._automation_client_enums import TokenType
from ._automation_client_enums import WindowsUpdateClasses
from ._automation_client_enums import WorkerType
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Activity",
    "ActivityListResult",
    "ActivityOutputType",
    "ActivityParameter",
    "ActivityParameterSet",
    "ActivityParameterValidationSet",
    "AdvancedSchedule",
    "AdvancedScheduleMonthlyOccurrence",
    "AgentRegistration",
    "AgentRegistrationKeys",
    "AgentRegistrationRegenerateKeyParameter",
    "AutomationAccount",
    "AutomationAccountCreateOrUpdateParameters",
    "AutomationAccountListResult",
    "AutomationAccountUpdateParameters",
    "AzureQueryProperties",
    "Certificate",
    "CertificateCreateOrUpdateParameters",
    "CertificateListResult",
    "CertificateUpdateParameters",
    "Connection",
    "ConnectionCreateOrUpdateParameters",
    "ConnectionListResult",
    "ConnectionType",
    "ConnectionTypeAssociationProperty",
    "ConnectionTypeCreateOrUpdateParameters",
    "ConnectionTypeListResult",
    "ConnectionUpdateParameters",
    "ContentHash",
    "ContentLink",
    "ContentSource",
    "Credential",
    "CredentialCreateOrUpdateParameters",
    "CredentialListResult",
    "CredentialUpdateParameters",
    "DeletedAutomationAccount",
    "DeletedAutomationAccountListResult",
    "Dimension",
    "DscCompilationJob",
    "DscCompilationJobCreateParameters",
    "DscCompilationJobListResult",
    "DscConfiguration",
    "DscConfigurationAssociationProperty",
    "DscConfigurationCreateOrUpdateParameters",
    "DscConfigurationListResult",
    "DscConfigurationParameter",
    "DscConfigurationUpdateParameters",
    "DscMetaConfiguration",
    "DscNode",
    "DscNodeConfiguration",
    "DscNodeConfigurationCreateOrUpdateParameters",
    "DscNodeConfigurationListResult",
    "DscNodeExtensionHandlerAssociationProperty",
    "DscNodeListResult",
    "DscNodeReport",
    "DscNodeReportListResult",
    "DscNodeUpdateParameters",
    "DscNodeUpdateParametersProperties",
    "DscReportError",
    "DscReportResource",
    "DscReportResourceNavigation",
    "EncryptionProperties",
    "EncryptionPropertiesIdentity",
    "ErrorResponse",
    "FieldDefinition",
    "GraphicalRunbookContent",
    "HybridRunbookWorker",
    "HybridRunbookWorkerCreateParameters",
    "HybridRunbookWorkerGroup",
    "HybridRunbookWorkerGroupCreateOrUpdateParameters",
    "HybridRunbookWorkerGroupsListResult",
    "HybridRunbookWorkerMoveParameters",
    "HybridRunbookWorkersListResult",
    "Identity",
    "Job",
    "JobCollectionItem",
    "JobCreateParameters",
    "JobListResultV2",
    "JobNavigation",
    "JobSchedule",
    "JobScheduleCreateParameters",
    "JobScheduleListResult",
    "JobStream",
    "JobStreamListResult",
    "Key",
    "KeyListResult",
    "KeyVaultProperties",
    "LinkedWorkspace",
    "LinuxProperties",
    "LogSpecification",
    "MetricSpecification",
    "Module",
    "ModuleCreateOrUpdateParameters",
    "ModuleErrorInfo",
    "ModuleListResult",
    "ModuleUpdateParameters",
    "NodeCount",
    "NodeCountProperties",
    "NodeCounts",
    "NonAzureQueryProperties",
    "Operation",
    "OperationDisplay",
    "OperationListResult",
    "OperationPropertiesFormatServiceSpecification",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionListResult",
    "PrivateEndpointProperty",
    "PrivateLinkResource",
    "PrivateLinkResourceListResult",
    "PrivateLinkServiceConnectionStateProperty",
    "ProxyResource",
    "PythonPackageCreateParameters",
    "PythonPackageUpdateParameters",
    "RawGraphicalRunbookContent",
    "Resource",
    "RunAsCredentialAssociationProperty",
    "Runbook",
    "RunbookAssociationProperty",
    "RunbookCreateOrUpdateDraftParameters",
    "RunbookCreateOrUpdateDraftProperties",
    "RunbookCreateOrUpdateParameters",
    "RunbookDraft",
    "RunbookDraftUndoEditResult",
    "RunbookListResult",
    "RunbookParameter",
    "RunbookUpdateParameters",
    "SUCScheduleProperties",
    "Schedule",
    "ScheduleAssociationProperty",
    "ScheduleCreateOrUpdateParameters",
    "ScheduleListResult",
    "ScheduleUpdateParameters",
    "Sku",
    "SoftwareUpdateConfiguration",
    "SoftwareUpdateConfigurationCollectionItem",
    "SoftwareUpdateConfigurationListResult",
    "SoftwareUpdateConfigurationMachineRun",
    "SoftwareUpdateConfigurationMachineRunListResult",
    "SoftwareUpdateConfigurationRun",
    "SoftwareUpdateConfigurationRunListResult",
    "SoftwareUpdateConfigurationRunTaskProperties",
    "SoftwareUpdateConfigurationRunTasks",
    "SoftwareUpdateConfigurationTasks",
    "SourceControl",
    "SourceControlCreateOrUpdateParameters",
    "SourceControlListResult",
    "SourceControlSecurityTokenProperties",
    "SourceControlSyncJob",
    "SourceControlSyncJobById",
    "SourceControlSyncJobCreateParameters",
    "SourceControlSyncJobListResult",
    "SourceControlSyncJobStream",
    "SourceControlSyncJobStreamById",
    "SourceControlSyncJobStreamsListBySyncJob",
    "SourceControlUpdateParameters",
    "Statistics",
    "StatisticsListResult",
    "SystemData",
    "TagSettingsProperties",
    "TargetProperties",
    "TaskProperties",
    "TestJob",
    "TestJobCreateParameters",
    "TrackedResource",
    "TypeField",
    "TypeFieldListResult",
    "UpdateConfiguration",
    "UpdateConfigurationNavigation",
    "Usage",
    "UsageCounterName",
    "UsageListResult",
    "UserAssignedIdentitiesProperties",
    "Variable",
    "VariableCreateOrUpdateParameters",
    "VariableListResult",
    "VariableUpdateParameters",
    "Watcher",
    "WatcherListResult",
    "WatcherUpdateParameters",
    "Webhook",
    "WebhookCreateOrUpdateParameters",
    "WebhookListResult",
    "WebhookUpdateParameters",
    "WindowsProperties",
    "AgentRegistrationKeyName",
    "AutomationAccountState",
    "AutomationKeyName",
    "AutomationKeyPermissions",
    "ContentSourceType",
    "CountType",
    "CreatedByType",
    "DscConfigurationState",
    "EncryptionKeySourceType",
    "GraphRunbookType",
    "GroupTypeEnum",
    "HttpStatusCode",
    "JobProvisioningState",
    "JobStatus",
    "JobStreamType",
    "LinuxUpdateClasses",
    "ModuleProvisioningState",
    "OperatingSystemType",
    "ProvisioningState",
    "ResourceIdentityType",
    "RunbookState",
    "RunbookTypeEnum",
    "ScheduleDay",
    "ScheduleFrequency",
    "SkuNameEnum",
    "SourceType",
    "StreamType",
    "SyncType",
    "TagOperators",
    "TokenType",
    "WindowsUpdateClasses",
    "WorkerType",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
